#!/usr/bin/python3

#	cve-manager : CVE management tool
#	Copyright (C) 2017-2025 Alexey Appolonov
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

import subprocess
from os import devnull
from re import sub as re_sub

MYSQL_ERR_CODE_TEMPLATE = r'^\[[^\]]*\]\s*'

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Form an error message based on a given command and a completed process

def ErrMsg(cmd, sproc):

	err = ''

	if not sproc or sproc.returncode != 0:
		err = f'{cmd[0]} command failed'
		if sproc:
			return f'{err}, return code {sproc.returncode}'

	return err

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Form a DB dumping command

def MySQLDumpCmd(mysql_config, ignored_tables={}):

	usr = mysql_config['user']
	pwd = mysql_config['password']
	db  = mysql_config['database']

	return ['mysqldump', f'--user={usr}', f'--password={pwd}', db] + \
		[f'--ignore-table={db}.{t}' for t in ignored_tables]

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Form a DB restoring command

def MySQLRestCmd(mysql_config):

	dump = MySQLDumpCmd(mysql_config)

	return ['mysql'] + dump[1:]

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Dump MySQL database to specified file using specified configuration

def MySQLDump(file_path, mysql_config, ignored_tables={}):

	cmd = MySQLDumpCmd(mysql_config, ignored_tables)
	err = ''
	sproc = None

	try:
		# Running formed command
		with open(file_path, 'w') as f:
			sproc = subprocess.run(cmd, stdout=f,
				# Hiding error and warning messages
				stderr=open(devnull, 'w'))
	except Exception as exc:
		err = re_sub(MYSQL_ERR_CODE_TEMPLATE, '', str(exc))

	if err:
		return err

	return ErrMsg(cmd, sproc)

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Restore MySQL database from specified file using specified configuration,
# return error message in case of a failure

def MySQLRestore(file_path, mysql_config):

	cmd = MySQLRestCmd(mysql_config)
	err = ''
	sproc = None

	try:
		# Running formed command
		with open(file_path, 'r') as f:
			sproc = subprocess.run(cmd, stdin=f,
				# Hiding the output as well as error and warning messages
				stdout=open(devnull, 'w'),
				stderr=subprocess.STDOUT)
	except Exception as exc:
		err = re_sub(MYSQL_ERR_CODE_TEMPLATE, '', str(exc))

	if err:
		return err

	return ErrMsg(cmd, sproc)
